//HHAssess.c
#include "option.h"
#include "stock.h"
#include "portfolio.h"
#include "plotting.h"
#include "falgorithm.h"

#include <cmath>
#include <iostream>
#include <iomanip.h>
#include <fstream>
#include <vector>
using namespace std;

class HHAssess
{
 public:
  double St0,interest,sigma;
  int j_H,R;
  int N;
  vector<double> times;
  int N_H;
  vector<double> Htimes;
  int N_BS;
  vector<double> BStimes;
  int M,seed;

  int callNum, putNum;
  vector<double> SPrice,amount,Stime;

  int hedgingMethod, errorOption; double assessTime;

  void readParameters();
  void readOutputOptions();
  void writeOutput(vector< vector<double> > outputs);
  void plottingOutput(vector< vector<double> > outputs);
};

int main(int argc, char **argv)
{
  HHAssess hs;
  hs.readParameters();
  hs.readOutputOptions();

  Stock stock(hs.St0, hs.interest, hs.sigma);

  vector<EuCall> ecs;
  vector<EuPut> eps;

  for(int i=0;i<hs.callNum;i++){
    EuCall ec(stock, hs.Stime[i], hs.SPrice[i], hs.amount[i]);
    ecs.push_back(ec);  
  }
  for(int i=hs.callNum;i<hs.putNum+hs.callNum;i++){
    EuPut ep(stock, hs.Stime[i], hs.SPrice[i], hs.amount[i]);  
    eps.push_back(ep); 
  }

  Portfolio p(ecs,eps);
  vector<double> bsps,hdps,hhps;
  vector<double> sampleHH,sampleBSP,sampleHD,samplePrices;
  vector< vector<double> > outputs;
    
  if(hs.M==1){
    Path path = stock.simulatePath(hs.times,hs.seed);
    hdps = p.sampleHoldingPortfolio(path);
    Path Hpath = path.subPath(hs.Htimes);
    hhps = p.sampleHHPayoff(Hpath,hs.j_H,hs.R);
    Path BSpath = path.subPath(hs.BStimes);
    bsps = p.sampleBSPortfolio(BSpath);
    
    samplePrices = path.prices;
    sampleHD = hdps;
    int j = -1,k = -1;
    for(int i=0;i<=hs.N;i++){
      if((int)(hs.BStimes[j+1]*100) == (int)(hs.times[i]*100))
	j++;
      if((int)(hs.Htimes[k+1]*100) == (int)(hs.times[i]*100))
	k++;
      sampleBSP.push_back(bsps[j]);
      sampleHH.push_back(hhps[k]);	
    }
    outputs.push_back(path.times);
  }else{
    for(int i=0;i<hs.M;i++){
      Path path = stock.simulatePath(hs.times,i+1);
      samplePrices.push_back(path.prices[path.timeIndex(hs.assessTime)]);
      hdps = p.sampleHoldingPortfolio(path);
      sampleHD.push_back(hdps[path.timeIndex(hs.assessTime)]);
      
      if(hs.hedgingMethod == 0){
	Path Hpath = path.subPath(hs.Htimes);
	hhps = p.sampleHHPayoff(Hpath,hs.j_H,hs.R);
	sampleHH.push_back(hhps[Hpath.timeIndex(hs.assessTime)]);
	
	Path BSpath = path.subPath(hs.BStimes);
	bsps = p.sampleBSPortfolio(BSpath);
	sampleBSP.push_back(bsps[BSpath.timeIndex(hs.assessTime)]);
      }else if(hs.hedgingMethod == 1){
	Path Hpath = path.subPath(hs.Htimes);
	hhps = p.sampleHHPayoff(Hpath,hs.j_H,hs.R);
	sampleHH.push_back(hhps[Hpath.timeIndex(hs.assessTime)]);
      }else{
	Path BSpath = path.subPath(hs.BStimes);
	bsps = p.sampleBSPortfolio(BSpath);
	sampleBSP.push_back(bsps[BSpath.timeIndex(hs.assessTime)]);
      }
    }

  }
  if(hs.M!=1){
    if(hs.hedgingMethod == 0)
      sortVectors(samplePrices,sampleBSP, sampleHH,sampleHD);
    else if( hs.hedgingMethod == 1 )
      sortVectors(samplePrices,sampleHH,sampleHD);
    else
      sortVectors(samplePrices,sampleBSP, sampleHD);
  }

  if(hs.M != 1)
    outputs.push_back(samplePrices);

  outputs.push_back(sampleHD);
  
  if(hs.hedgingMethod == 1)
    outputs.push_back(sampleHH);
  else if( hs.hedgingMethod == -1 )
    outputs.push_back(sampleBSP);
  else{
    outputs.push_back(sampleBSP);
    outputs.push_back(sampleHH);
  }

  hs.writeOutput(outputs);
  hs.plottingOutput(outputs);
  return 0;
}


void HHAssess::writeOutput(vector< vector<double> > outputs)
{  
  double ehh=0.0,ebs=0.0;
  ofstream result("./Output/result.txt",ios::out);
  result<<setprecision(10);

  result<<"double St0,interest,sigma"<<endl;
  result<<"int j_H,R"<<endl;
  result<<"int N"<<endl;   
  result<<"vector<double> times"<<endl;
  result<<"int N_H"<<endl;
  result<<"vector<double> Htimes"<<endl;
  result<<"int N_BS"<<endl;
  result<<"vector<double> BStimes"<<endl;
  result<<"int M,seed"<<endl;
  result<<"int callNum,putNum"<<endl;
  result<<"vector<double> SPrice,amount,Stime"<<endl;
  result<<"int hedgingMethod, errorOption, double assessTime"<<endl<<endl;
  
  result<<St0<<" "<<interest<<" "<<sigma<<endl;
  result<<j_H<<" "<<R<<endl;
  result<<N<<endl;
  for(int i=0;i<=N;i++)
    result<<times[i]<<" ";
  
  result<<endl<<N_H<<endl; 
  for(int i=0;i<=N_H;i++)
    result<<Htimes[i]<<" ";
  
  result<<endl<<N_BS<<endl;
  for(int i=0;i<=N_BS;i++)
    result<<BStimes[i]<<" ";

  result<<endl<<M<<" "<<seed<<endl;


  result<<callNum<<" "<<putNum<<endl;
  for(int i=0;i<callNum+putNum;i++)
    result<<SPrice[i]<<" "<<amount[i]<<" "<<Stime[i]<<endl;

  result<<hedgingMethod<<" "<<errorOption<<" "<<assessTime<<endl<<endl;

  if(hedgingMethod == 1){
    if(errorOption==1)
      result<<"samplePrices sampleHH-sampleHD"<<endl;     
    else
      result<<"samplePrices  sampleHD  sampleHH"<<endl;  
    for(int i=0;i<outputs[0].size();i++){
      if(errorOption==1)
	result<<outputs[0][i]<<"    "<<outputs[2][i]-outputs[1][i]<<endl;
      else
	result<<outputs[0][i]<<"    "<<outputs[1][i]<<"    "<<outputs[2][i]<<endl;	
      
      ehh += pow(outputs[2][i]-outputs[1][i],2.0);
    }
    result<<"Average HH error = "<<sqrt(ehh/outputs[0].size())<<endl;
  }
  else if(hedgingMethod == -1){
    if(errorOption==1)
      result<<"samplePrices sampleBSP-sampleHD"<<endl;     
    else
      result<<"samplePrices  sampleHD  sampleBSP"<<endl;  
    for(int i=0;i<outputs[0].size();i++){
      if(errorOption==1)
	result<<outputs[0][i]<<"    "<<outputs[2][i]-outputs[1][i]<<endl;
      else
	result<<outputs[0][i]<<"    "<<outputs[1][i]<<"    "<<outputs[2][i]<<endl;	
      
      ebs += pow(outputs[2][i]-outputs[1][i],2.0);
    }
    result<<"Average BSP error = "<<sqrt(ebs/outputs[0].size())<<endl;
  }
  else{
    if(errorOption==1)
      result<<"samplePrices  sampleBSP-sampleHD  sampleHH-sampleHD"<<endl;     
    else
      result<<"samplePrices  sampleHD  sampleBSP  sampleHH"<<endl; 
    for(int i=0;i<outputs[0].size();i++){
      if(errorOption==1)
	result<<outputs[0][i]<<"    "<<outputs[2][i]-outputs[1][i]<<"    "
	      <<outputs[3][i]-outputs[1][i]<<endl;
      else
	result<<outputs[0][i]<<"    "<<outputs[1][i]<<"    "<<outputs[2][i]
	      <<"    "<<outputs[3][i]<<endl;
      
      ebs += pow(outputs[2][i]-outputs[1][i],2.0);
      ehh += pow(outputs[3][i]-outputs[1][i],2.0);
    }
       
    result<<"Average BSP error = "<<sqrt(ebs/outputs[0].size())<<endl;
    result<<"Average HH  error = "<<sqrt(ehh/outputs[0].size())<<endl;
  }

  ofstream priceResult("./Output/priceResult.txt",ios::out);
  ofstream HDResult("./Output/HDResult.txt",ios::out);
  ofstream HHResult("./Output/HHResult.txt",ios::out);
  ofstream BSPResult("./Output/BSPResult.txt",ios::out);
  
  for(int i=0;i<outputs[0].size();i++){
    priceResult<<outputs[0][i]<<endl;
    HDResult<<outputs[1][i]<<endl;
    if(hedgingMethod == 0){
      BSPResult<<outputs[2][i]<<endl;
      HHResult<<outputs[3][i]<<endl;
    }else if(hedgingMethod == 1)
      HHResult<<outputs[2][i]<<endl;
    else
      BSPResult<<outputs[2][i]<<endl;
  }
}

void HHAssess::plottingOutput(vector< vector<double> > outputs)
{
  vector<double> xvector = outputs[0];
  vector< vector<double> > yvectors;
  char *names[20];

  if(errorOption==0){
    yvectors.push_back(outputs[1]);
    if(hedgingMethod == -1){
      yvectors.push_back(outputs[2]);
      names[1] = "BS porfolio";
    }
    else if(hedgingMethod == 1){
      yvectors.push_back(outputs[2]);
      names[1] = "HH Payoff";
    }
    else{
      yvectors.push_back(outputs[2]);
      yvectors.push_back(outputs[3]);
      names[1] = "BS porfolio";
      names[2] = "HH Payoff";
    }
    names[0] = "Hold Payoff";
  }
  else{
    vector<double> v1,v2;
    for(int i=0;i<outputs[0].size();i++){      
      if(hedgingMethod == 0){
	v1.push_back(outputs[2][i]-outputs[1][i]);
	v2.push_back(outputs[3][i]-outputs[1][i]);
      }else
      	v1.push_back(outputs[2][i]-outputs[1][i]);
    }
    if(hedgingMethod == 0){
      yvectors.push_back(v1);
      yvectors.push_back(v2);
    }
    else
      yvectors.push_back(v1);

    if(hedgingMethod == 0){
      names[0] = "BS porfolio";
      names[1] = "HH Payoff";
    }else if(hedgingMethod == 1)
      names[0] = "HH Payoff";
    else
      names[0] = "BS porfolio";
  }

  vectorPlotting(yvectors,xvector,names);
}

void HHAssess::readParameters()
{
  double atime,temp;
  ifstream param("parameter.txt",ios::in);

  param>>St0;param>>interest;param>>sigma;

  param>>j_H; param>>R;

  param>>N;
  for(int i=0;i<=N;i++){
    param>>atime;
  if(i>0 && atime<=times[times.size()-1])
    {
      cout<<"Input error: General times must be ordered!"<<endl;
      exit(0);
    }
    times.push_back(atime);
  }
  if(N!=times.size()-1 || N<1)
    {
      cout<<"Input error: N must be equal to the number of times!"<<endl;
      exit(0);
    }
  param>>N_H;
  for(int i=0;i<=N_H;i++){
    param>>atime;
    Htimes.push_back(atime);
  }
  if(N_H!=Htimes.size()-1 || N_H<1)
    {
      cout<<"Input error: N_H must be equal to the number of HH times!"<<endl;
      exit(0);
    }
  param>>N_BS;
  for(int i=0;i<=N_BS;i++){
    param>>atime;
    BStimes.push_back(atime);
  }
  if(N_BS!=BStimes.size()-1 || N_BS<1)
    {
      cout<<"Input error: N_BS must be equal to the number of BS times!"<<endl;
      exit(0);
    }
  param>>M;param>>seed;
  if(M<1){
    cout<<"Input error: No path for simulation! M should greater than 0."<<endl;
    exit(0);
  }

  param>>callNum;param>>putNum;
  for(int i=0;i<putNum+callNum;i++){
    param>>temp;SPrice.push_back(temp);
    param>>temp;amount.push_back(temp); 
    param>>temp;Stime.push_back(temp);
  }
}

void HHAssess::readOutputOptions(){
  ifstream param2("outputops.txt",ios::in);
  param2>>hedgingMethod;
  param2>>errorOption;
  param2>>assessTime;
}
