//falgorithm.c
#include "falgorithm.h"

//#include "float.h"
//#include "math.h"
//#include "assert.h"
#include <cmath>
#include <vector>
using namespace std;

// BEGIN code1
double cumulative(double z)
{
  return 0.5 * ( 1 + erf( z/sqrt(2.0) ) );
}
// END code1
double inverseCumulative(double u){

  assert( 0 < u && u < 1 );
    
  // Coefficients for the rational approximation.
  const double
    a1 = -3.969683028665376e+01,
    a2 =  2.209460984245205e+02,
    a3 = -2.759285104469687e+02,
    a4 =  1.383577518672690e+02,
    a5 = -3.066479806614716e+01,
    a6 =  2.506628277459239e+00;
    
  const double
    b1 = -5.447609879822406e+01,
    b2 =  1.615858368580409e+02,
    b3 = -1.556989798598866e+02,
    b4 =  6.680131188771972e+01,
    b5 = -1.328068155288572e+01;
    
  const double
    c1 = -7.784894002430293e-03,
    c2 = -3.223964580411365e-01,
    c3 = -2.400758277161838e+00,
    c4 = -2.549732539343734e+00,
    c5 =  4.374664141464968e+00,
    c6 =  2.938163982698783e+00;
    
  const double
    d1 =  7.784695709041462e-03,
    d2 =  3.224671290700398e-01,
    d3 =  2.445134137142996e+00,
    d4 =  3.754408661907416e+00;
    
  // Limits of the approximation region.
  const double
    u_low   = 0.02425,
    u_high  = 1.0 - u_low;

  register double z, r;
  if( u < u_low ){
    z = sqrt(-2.0*log(u));
    z = (((((c1*z+c2)*z+c3)*z+c4)*z+c5)*z+c6) / ((((d1*z+d2)*z+d3)*z+d4)*z+1.0);
  }  else if( u <= u_high ){
    z = u - 0.5;
    r = z*z;
    z = (((((a1*r+a2)*r+a3)*r+a4)*r+a5)*r+a6)*z / (((((b1*r+b2)*r+b3)*r+b4)*r+b5)*r+1.0);
  } else {
    z = sqrt(-2.0*log(1.0-u));
    z = -(((((c1*z+c2)*z+c3)*z+c4)*z+c5)*z+c6) /  ((((d1*z+d2)*z+d3)*z+d4)*z+1.0);
  }

  return z;
}
// BEGIN code3
vector<double> partitiate(int J_H)
{
  double spot;
  vector<double> spots;
  int length = 1<<J_H;

  for(int i=1; i < length/2; i++)
    {
      spot = inverseCumulative(1.0*i/length);
      spots.push_back(spot);
    } 

  spots.push_back(0); //C_{2^{J-1}-1}^J
  
  for(int i = 1; i < length/2; i++)
    {
      spot = (-1) * spots[length/2 - 1 - i];
      spots.push_back(spot);
    } 
  return spots;
}
// END code3
vector<double> matrixProduct(vector<double> rp1,vector<double> rp2)
{
  int i,k,length1,length2;
  vector<double> rp;

    length1 = rp1.size();
    length2 = rp2.size();
    for(i=0;i<length1;i++)
      for(k=0;k<length2;k++)
	rp.push_back(rp1[i] * rp2[k]);
    return rp;
}

// BEGIN code4
vector<indexItem>  getIndexItems(vector<double> vector1){
  int length = vector1.size();

  indexItem aIndexItem;
  vector<indexItem> indexVector;
 
 for(int i=0; i < length; i++)
    {
      aIndexItem.item = vector1[i];
      aIndexItem.index = i; 
      indexVector.push_back(aIndexItem);
    } 
 return indexVector;
}
// END code4
// BEGIN code5_0
vector<indexItem> merge(vector<indexItem> vector1, vector<indexItem>  vector2)
{
  vector<indexItem>::iterator first,first1,first2,last1,last2;
  vector<indexItem> temp(vector1.size() + vector2.size());
  
  first = temp.begin();
  first1 = vector1.begin();
  last1 = vector1.end();
  first2 = vector2.begin();
  last2 = vector2.end();
  
  while( (first1 < last1) && (first2 < last2))
    {
      if( fabs(first1->item) > fabs(first2->item))
        *(first++) = *(first1++);
      else
        *(first++) = *(first2++);
    }
  while(first1 < last1)
      *(first++) = *(first1++);
  
  while( first2 < last2 )
    *(first++) = *(first2++);

  return temp;
}
// END code5_0

// BEGIN code5
vector<indexItem>  mergeSort(vector<indexItem>  vectorCompress)
{
  int i,half,whole;
  vector<indexItem> vector1, vector2;
  vector<indexItem> sortedVector1,sortedVector2;	
  
  // compressItem temp;
  if(vectorCompress.size() > 1)
     {
       whole = vectorCompress.size();
       half = whole/2;
       
       for(i=0;i<half;i++)
	 vector1.push_back( vectorCompress[i] );
       for(i=half;i<whole;i++)
	 vector2.push_back( vectorCompress[i] );
       
       sortedVector1 = mergeSort(vector1);
       sortedVector2 = mergeSort(vector2);
       return merge(sortedVector1,sortedVector2);
     }
   else
     return vectorCompress;
}
// END code5
// BEGIN code6
vector<int> sortVector(vector<double> &vector1){
  vector<indexItem> indexedVector;
  vector<int> indexes;

  indexedVector = getIndexItems(vector1);
  indexedVector = mergeSort(indexedVector);
  
  int length = vector1.size();
  for(int i=0;i<length;i++){
    vector1[i] = indexedVector[length-1-i].item;
    indexes.push_back( indexedVector[length-1-i].index);
  }

  return indexes;
}
// END code6
void sortVectors(vector<double> &vector0,vector<double> &vector1){
   vector<int> indexes = sortVector(vector0);
   reorderVector(vector1,indexes);
}

void sortVectors(vector<double> &vector0,vector<double> &vector1, vector<double> &vector2){
   vector<int> indexes = sortVector(vector0);
   reorderVector(vector1,indexes);
   reorderVector(vector2,indexes);
}

void sortVectors(vector<double> &vector0,vector<double> &vector1, vector<double> &vector2,
		 vector<double> &vector3){
   vector<int> indexes = sortVector(vector0);
   reorderVector(vector1,indexes);
   reorderVector(vector2,indexes);
   reorderVector(vector3,indexes);
}
// BEGIN code8
void sortVectors(vector<double> &vector0,vector<double> &vector1, vector<double> &vector2,
		 vector<double> &vector3,vector<double> &vector4){
   vector<int> indexes = sortVector(vector0);
   reorderVector(vector1,indexes);
   reorderVector(vector2,indexes);
   reorderVector(vector3,indexes);
   reorderVector(vector4,indexes);
}
// END code8
// BEGIN code7
void reorderVector(vector<double> &vector1,vector<int> indexes){
  vector<double> tempVector;
  int length = vector1.size();
  for(int i=0;i<length;i++)
    tempVector.push_back(vector1[i]);
  for(int i=0;i<length;i++)
    vector1[i]= tempVector[indexes[i]];
}
// END code7

int getIndex(vector<double> spots,double value){
  for(int i=0; i<spots.size(); i++){
    if(value < spots[i])
      return i;
  }
  return spots.size();
}
