//haar.c

#include "haar.h"
#include "falgorithm.h"

#include <cmath>
#include <iterator>
#include <iostream>
#include <vector>
using namespace std;

Haar::Haar(vector<double> theItems,int the_J, int the_j){
  haarItems = theItems;
  j = the_j;
  J = the_J;
}

Haar::Haar(){
  vector<double> items;
  Haar(items, 0, 0);
}
// BEGIN code1
Haar Haar::oneStepHaarAnalysis(){
  int p=1 << (j-1);
  Haar OV(haarItems,J,j-1);

  vector<double>::iterator OV_first_ptr = OV.haarItems.begin();  
  vector<double>::iterator OV_second_ptr = OV_first_ptr + p;

  double d;
  for(int i=0; i < p; i++)
    {
      d = (1/sqrt(2.0)) * ( haarItems[2*i + 1] - haarItems[2*i] );
      *(OV_second_ptr++) = d;
      *(OV_first_ptr++)  = sqrt(2.0) *  haarItems[2*i] + d;
    }
  return OV;
}

//transform haar vector numberOfCalls steps by call oneStepAnalysis

Haar Haar::haarAnalysis(int numberOfCalls){
  if(numberOfCalls > j){
    cout<<"Too much times of Analysis"<<endl;
    exit(1);
  }
    
  Haar OV(haarItems,J,j);
  
  for(int i=0;i < numberOfCalls; i++)
      OV = OV.oneStepHaarAnalysis();

  return OV;
}
// END code1
void Haar::printHaarVector(){
  for(int i=0; i < haarItems.size(); i++)
      cout<<haarItems[i]<<"  ";
}

double Haar::energy(){
  double energy = 0.0;
  for(int i=0; i < haarItems.size(); i++)
    energy += pow(haarItems[i],2.0);
  return energy;
}

// BEGIN code2
Haar Haar::oneStepHaarSynthesis(){
  Haar OV(haarItems,J,j+1);
  vector<double>::iterator OV_ptr = OV.haarItems.begin();  

  int p = 1<<j;

  double d;
  for(int i=0; i < p; i++)
    {
      d = haarItems[p + i];
      *OV_ptr = ( 1/sqrt(2.0) ) * ( haarItems[i] - d );
      *(OV_ptr+1)  = *OV_ptr + sqrt(2.0) *  d;
      OV_ptr +=2;
    }  
  return OV;
}

//transform vector numberOfCalls steps by call oneStepAnalysis
Haar Haar::haarSynthesis(int numberOfCalls){
  if(numberOfCalls > J-j){
    cout<<"Two much times of systhesis"<<endl;
    exit(1);
  }

  Haar OV(haarItems,J,j);
  
  for(int i=0;i < numberOfCalls; i++)
    OV = OV.oneStepHaarSynthesis();
  return OV;
}
// END code2

// BEGIN code3
Haar Haar::RCompression(int R){
  vector<indexItem> indexVector = getIndexItems(haarItems);
 
  indexVector = mergeSort(indexVector); 

  vector<double> v(1<<J,0.0);
  Haar OV(v,J,j);
  vector<double>::iterator OV_ptr;

  for(int i=0; i < R; i++)
    {
      OV_ptr = OV.haarItems.begin();
      OV_ptr += indexVector[i].index;
      *OV_ptr = indexVector[i].item;
    }
  return OV;
}

Haar Haar::S0RCompression(int R){
  haarItems[0] = 0;
  return RCompression(R);
}
// END code3
Haar Haar::thresholdCompression(double threshold,int &Rt){
  vector<indexItem> indexVector = getIndexItems(haarItems);
 
  indexVector = mergeSort(indexVector); 

  vector<double> v(1<<J,0.0);
  Haar OV(v,J,j);
  vector<double>::iterator OV_ptr;

  double criticalEnergy = energy()  * threshold;
  double thresholdEnergy = 0;
  int i,length = 1 << J;
  for(i=0; i < length; i++)
    {
      thresholdEnergy += pow(indexVector[i].item,2.0);
      OV_ptr = OV.haarItems.begin();
      OV_ptr += indexVector[i].index;
      *OV_ptr = indexVector[i].item;
      if( thresholdEnergy >= criticalEnergy )
	break;
    }
  Rt = i+1;
  return OV;
}

