/*
plotting.c
Plotting one or multiple lines, use vector of Points
*/
#include <g2_X11.h>
#include <g2_PS.h>
#include <cmath>
#include <unistd.h>
#include <stdio.h>
#include <vector>
#include "plotting.h"
// BEGIN code1
void plottingCurve(vector<Point> vector, int id,double coeff_ax,
		   double coeff_ay,double coeff_bx,double coeff_by)
{
  int i,length;

  length = vector.size();
  double points[length*2];

  for(i=0; i < length; i++)
    {
      points[2*i] = coeff_ax * vector[i].x + coeff_bx;
      points[2*i+1] = coeff_ay * vector[i].y + coeff_by;
    }
  g2_poly_line(id,length,points);
}
// END code1
/*
Compute the extreme y values for the vector, and assuming the first point has the minimum x value
*/
void extremeY(double& max_y ,double& min_y,const vector<Point> vector)
{
  int i,length = vector.size();

  max_y=vector[0].y;
  min_y=vector[0].y;

  for(i=1; i < length; i++)
    {
      if(vector[i].y > max_y)
	max_y = vector[i].y;
      if(vector[i].y < min_y)
	min_y = vector[i].y;
    }
}
/*
compare compare 2 groups of extreme values,
return by reference
*/
void valuesComparing(double& max_x, double& min_x,double& max_y ,double& min_y,double max_x1, double min_x1,double max_y1 ,double min_y1)
{
  if(max_x1 > max_x)
    max_x = max_x1;
  if(max_y1 > max_y)
    max_y = max_y1;

  if(min_x1 < min_x)
    min_x = min_x1;
  if(min_y1 < min_y)
    min_y = min_y1;
}
// BEGIN code2
void extremeValues(double& max_x, double& min_x,double& max_y ,
		   double& min_y,const vector< vector<Point> > vectors)
{
  double max_y1,min_y1;

  max_x = vectors[0][vectors[0].size() - 1].x;
  min_x = vectors[0][0].x;
  extremeY(max_y1,min_y1, vectors[0]);
  max_y = max_y1;
  min_y = min_y1;

  for(int i=0; i < vectors.size(); i++){
    extremeY(max_y1,min_y1, vectors[i]);
    valuesComparing(max_x,min_x,max_y ,min_y, 
		    vectors[i][vectors[i].size() - 1].x, vectors[i][0].x, max_y1, min_y1);
  }  
}
// END code2 

// BEGIN code3
void coefficient(double& coeff_ax, double& coeff_bx,double& coeff_ay ,
		 double& coeff_by,double max_x,double min_x,double max_y,double min_y)
{
  coeff_ax = ( MAX_PX - 2 * EDGE )/(max_x - min_x);
  coeff_bx = ( max_x * (EDGE+MOD1) - min_x * (MAX_PX - EDGE+MOD1) )/(max_x - min_x);
  
  coeff_ay = ( MAX_PY - 2 * EDGE - MOD2 )/(max_y - min_y);
  coeff_by = ( max_y * EDGE - min_y * (MAX_PY - EDGE - MOD2) )/(max_y - min_y);
}
// END code3

void drawRectangle(int id,double max_x,double min_x,double max_y,double min_y)
{
  char str[20];
  int i, interval;

  g2_rectangle(id,EDGE+MOD1,EDGE,MAX_PX - EDGE+MOD1,MAX_PY - EDGE);

  //draw string
  if(min_x-(double)((int)min_x)>0)
    sprintf(str,"%-7.1f",min_x);
  else
    sprintf(str,"%-7d",(int)min_x);
  g2_string(id,EDGE+MOD1,EDGE -15,str);
  
  if(max_x-(double)((int)max_x)>0)
    sprintf(str,"%-7.1f",max_x);
  else
    sprintf(str,"%7d",(int)max_x);
  g2_string(id,MAX_PX - (EDGE-MOD1) - 25,EDGE - 15,str);

  sprintf(str,"%7.4f",min_y);
  g2_string(id,TE,EDGE,str);
  sprintf(str,"%7.4f",max_y);
  g2_string(id,TE,MAX_PY - EDGE - MOD2,str);

  interval =(MAX_PY - 2*EDGE - MOD2)/10;

  for(i=1;i<10;i++)
     g2_line(id,EDGE+MOD1,EDGE + i * interval, EDGE+MOD1+5,EDGE + i * interval);
  g2_line(id,EDGE+MOD1, EDGE + i * interval, MAX_PX - EDGE+MOD1,EDGE + i * interval);
}

void vectorPlotting(const vector< vector<Point> > vectors,char *names[])
{
  vectorPlotting(0.0,vectors,names,0);
}

void vectorPlotting(const double exactValue,const vector < vector<Point> > vectors,char *names[])
{
  vectorPlotting(exactValue,vectors,names,1);
}

void defineColors(int id,vector<int> &color){
  color.push_back( g2_ink(id,1.0,0.0,0.0) );
  color.push_back( g2_ink(id,0.0,0.0,1.0) );
  color.push_back( g2_ink(id,1.0,0.0,1.0) ); 
  color.push_back( g2_ink(id,0.0,1.0,1.0) );
   color.push_back( g2_ink(id,1.0,0.0,1.0) );
  color.push_back( g2_ink(id,0.0,1.0,0.0) );
  color.push_back( g2_ink(id,0.5,0.0,0.0) );
  color.push_back( g2_ink(id,0.0,0.5,0.0) );
  color.push_back( g2_ink(id,0.0,0.0,0.5) );
  color.push_back( g2_ink(id,0.5,0.5,0.0) );
  color.push_back( g2_ink(id,0.5,0.0,0.5) );
}
// BEGIN code4
void vectorPlotting(const double exactValue,const vector < vector<Point> > vectors,
		    char *names[],int style)
{
  double max_x,max_y,min_x,min_y;
  double coeff_ax, coeff_bx,coeff_ay, coeff_by;

  int id,id_PS, id_X11;
  char *filename = "./Output/result.ps",str[20];

  double px_0,py_0;
  double px_1,py_1;

  vector<int> color;
  double dashes[3];

  extremeValues(max_x,min_x,max_y,min_y,vectors);
  //max_y += 0.15 * (max_y - min_y); 
   
  coefficient(coeff_ax,coeff_bx,coeff_ay,coeff_by,max_x,min_x,max_y,min_y);

  //open devices
  id = g2_open_vd();
  id_X11 = g2_open_X11((int)MAX_PX,(int)MAX_PY);
  id_PS=g2_open_PS(filename,PAPERSIZE,g2_PS_land);
  //attach x11 and ps to visual device
  g2_attach(id,id_X11);
  g2_attach(id,id_PS);

   //draw rectangle  
  drawRectangle(id,max_x,min_x,max_y,min_y);

  //draw the exact line
  if(style == 1){
    px_0 = coeff_ax * min_x + coeff_bx;
    px_1 = coeff_ax * max_x + coeff_bx;
    py_0 = coeff_ay * exactValue + coeff_by;
    py_1 = py_0;
    g2_line(id,px_0,py_0,px_1,py_1);
    if(exactValue == 0)
      g2_string(id,px_0 - 15,py_0,"0");
    else
      {
	sprintf(str,"exact value=%-8.6f",exactValue);
	g2_string(id,px_1 - 160,py_0+10,str);
      }
  }

  g2_set_line_width(id,2);

  defineColors(id_PS,color);
  defineColors(id_X11,color);
 
  dashes[0]=15.0;
  dashes[1]=5.0;
  dashes[2]=5.0;

  for(int i=0; i < vectors.size(); i++){
    g2_set_dash(id,i*5,dashes);
    g2_pen(id,color[i]);
    
    g2_line(id,(2 + i*4) * EDGE, MAX_PY-EDGE-MOD2*0.5,(3 + i*4) * EDGE, MAX_PY-EDGE-MOD2*0.5);  
    g2_string(id,(3.2 + i*4) * EDGE, MAX_PY-EDGE-MOD2*0.7, names[i]);

    plottingCurve(vectors[i], id,coeff_ax,coeff_ay,coeff_bx,coeff_by);
  }  
  
  g2_flush(id);
  getchar();
  g2_close(id);
}
// END code4

void vectorPlotting(const double exactValue,const vector < vector<double> > yvectors,const vector<double> xvector,char *names[])
{
  vector< vector<Point> > pointVectors;
  getPointVector(pointVectors,yvectors,xvector);
  vectorPlotting(exactValue,pointVectors,names);
}

void vectorPlotting(const vector < vector<double> > yvectors,const vector<double> xvector,char *names[])
{
  vector< vector<Point> > pointVectors;
  getPointVector(pointVectors,yvectors,xvector);

  vectorPlotting(pointVectors,names);
}

void getPointVector(vector< vector<Point> > &pointVectors,const vector< vector<double> > yvectors,const vector<double> xvector)
{
  Point point;
  vector<Point> pointVector(xvector.size());

  for(int i = 0; i < yvectors.size(); i++){
    for(int j = 0; j < xvector.size(); j++){
      point.x = xvector[j];
      point.y = yvectors[i][j];
      pointVector[j] = point;
    }
    pointVectors.push_back(pointVector);
  }
}

void readVector(vector<double> & approximationVector,vector<double> &MValues,ifstream &result,int length)
{
  int i;
  double MValue,price;
  approximationVector.clear();
  MValues.clear();
  for(i=0; i < length; i++)
    {
      result>>price;
      approximationVector.push_back(price);
    }
  for(i=0; i < length; i++)
    {
      result>>MValue;
      MValues.push_back(MValue);
    }
}

