#include "portfolio.h"
#include "stock.h"
#include "haar.h"
#include "option.h"
#include "falgorithm.h"
#include <cmath>
#include <iostream>
#include <vector>
using namespace std;

Portfolio::Portfolio(vector<EuCall> theEcs,vector<EuPut> theEps){
  ecs = theEcs;
  eps = theEps;
}

Portfolio::Portfolio(){
}
// BEGIN portfc1
vector<double> Portfolio::sampleBSPortfolio(Path path){
  vector<double> BSPs;
  double BSP=0;
  if(ecs.size()==0 && eps.size()==0){
    cout<<"No options in the porfolio error"<<endl;
    exit(1);
  }

  for(int i = 0; i < ecs.size(); i++){
    BSP += ecs[i].value(path.prices[0], 0.0);
  }
  for(int i = 0; i < eps.size(); i++){
    BSP += eps[i].value(path.prices[0], 0.0);
  }
  BSPs.push_back(BSP);   //black and schole value at t0

  double deltaT;
  double d1,phi,xi,epayoff; //expire payoff
  for(int i=1; i < path.times.size(); i++){
    deltaT = path.times[i] - path.times[i-1];
    phi = 0;
    epayoff = 0;
    
    for(int j = 0; j < ecs.size(); j++){
      if(path.times[i-1] < ecs[j].etime){
	d1 = ecs[j].get_d1(path.prices[i-1],path.times[i-1]);
	phi += ecs[j].get_phi(d1);
      }
      //else if(path.times[i-1] == ecs[j].etime)
      //epayoff += ecs[j].evalue(path.prices[i-1]);
      
    }
    
    for(int j = 0; j < eps.size(); j++){
      if(path.times[i-1] < eps[j].etime){
	d1 = eps[j].get_d1(path.prices[i-1],path.times[i-1]);
	phi += eps[j].get_phi(d1);
      }
      //else if(path.times[i-1] == eps[j].etime)
      //epayoff += eps[j].evalue(path.prices[i-1]);
    }
    
    xi = BSP - phi * path.prices[i-1];
    //xi = BSP - phi * path.prices[i-1] - epayoff;
    double interest;
    if(ecs.size()>0) //at least one option
      interest = ecs[0].stock.r;
    else
      interest = eps[0].stock.r;
    BSP = xi * exp(interest * deltaT) + phi * path.prices[i];

    BSPs.push_back(BSP);
  }
  return BSPs;  
}
// END portfc1

// BEGIN portfc2
vector<double> Portfolio::sampleHoldingPortfolio(Path path){
  vector< vector<double> > hdps;
  vector<double> HDP;
  HDP.assign(path.times.size(),0.0);

  if(ecs.size()==0 && eps.size()==0){
    cout<<"No options in the porfolio error"<<endl;
    exit(1);
  }
  
  for(int i = 0; i < ecs.size(); i++)
    hdps.push_back( ecs[i].holdingPortfolio(path) );

  for(int i = 0; i < eps.size(); i++)
    hdps.push_back( eps[i].holdingPortfolio(path) );

  for(int i = 0; i < hdps.size(); i++)
    for(int j = 0; j < hdps[i].size(); j++) 
       HDP[j] += hdps[i][j];
  return HDP;  
}
// END portfc2

// BEGIN portfc3
vector<double> Portfolio::sampleHHPayoff(Path path,int j_H,int R){
  vector<double> HHPs;
  double HHP=0;
  double Xi;
  vector<double> Xis;
  vector< vector<double> > payoffs;

  double interest;
  if(ecs.size()>0) //at least one option
    interest = ecs[0].stock.r;
  else
    interest = eps[0].stock.r;
  
  if(ecs.size()==0 && eps.size()==0){
    cout<<"No options in the porfolio error"<<endl;
    exit(1);
  }

  for(int i = 0; i < ecs.size(); i++){
    HHP += ecs[i].value(path.prices[0], 0.0);
  }
  for(int i = 0; i < eps.size(); i++){
    HHP += eps[i].value(path.prices[0], 0.0);
  }
  HHPs.push_back(HHP);
  //at this point HHP contains V_{t_0}(X^{t_0})
  //with this initialization we should take s_0[0] =0 
  
  //here we compute/load the input to Haar analysis
  //  HHP =0; //FOR DEBUGGING
  for(int k=0; k<path.times.size()-1; k++){ //start of big loop
    payoffs.clear();
    for(int i = 0; i < ecs.size(); i++){
      Xis.clear();
      if(path.times[k] < ecs[i].etime){
	Xis = ecs[i].HHPayoff(j_H,path.prices[k],path.times[k],path.times[k+1] 
			      - path.times[k]);
	payoffs.push_back(Xis);
      }
    }

    for(int i = 0; i < eps.size(); i++){
      Xis.clear();
      if(path.times[k] < eps[i].etime){
	Xis = eps[i].HHPayoff(j_H,path.prices[k],path.times[k],path.times[k+1] 
			      - path.times[k]);
	payoffs.push_back(Xis);
      }
    }

    //sum up contributions of all options
    Xis.clear();
    for(int i = 0; i < payoffs[0].size(); i++){
      Xi = 0;
      for(int j = 0; j < payoffs.size(); j++)
	Xi += payoffs[j][i];
      Xis.push_back(Xi);
    }

    Haar h(Xis,j_H,j_H);
    
    h = h.haarAnalysis(j_H);
     h = h.S0RCompression(R); //this compression sets s_0[0]=0
    //h = h.RCompression(R);//this compression DOES NOT  force s_0[0]=0 
    h = h.haarSynthesis(j_H);
    
    vector<double> spots = partitiate(j_H);
    int index = getIndex(spots,path.allJumps[k+1]);
    HHP = HHP * exp((path.times[k+1]-path.times[k]) * interest) + 
      h.haarItems[index];
    HHPs.push_back(HHP);
  }//end of big loop
  return HHPs;  
}
// END portfc3


