//stock.c
#include "stock.h"
#include "falgorithm.h"
#include <iostream>
#include <cmath>
#include <vector>
using namespace std;

Stock::Stock(double theSt0,double theR,double theSigma){
  St0 = theSt0;
  r = theR;
  sigma = theSigma;
}

Stock::Stock(){
  Stock(20.0, 0.1, 0.2);
}
// BEGIN code2
Path Stock::simulatePath(vector<double> times){
  double drawNumber,NDNumber;
  double deltaT;
  double ST = St0;
  vector<double> prices,allJumps;

  prices.push_back(ST);
  allJumps.push_back(1.0);
  for(int i=1;i<times.size();i++)
    {
      drawNumber=rand()/(double)(RAND_MAX);
      NDNumber = inverseCumulative(drawNumber);

      allJumps.push_back(NDNumber);
	deltaT = times[i] - times[i-1]; 
      ST = ST * exp( (r - pow(sigma,2.0)/2) * deltaT  +  NDNumber * sigma * sqrt(deltaT));
      prices.push_back(ST);
    }
  return Path(prices,allJumps,times);
}
// END code2

Path Stock::simulatePath(vector<double> times,int seed){
  srand(seed);
  return simulatePath(times);
}
// BEGIN code1
vector<double> Stock::oneStepUnderlying1(int J_H,double currentPrice,double deltaT){
  vector<double> underlying;

  double nu = r - pow(sigma,2.0)/2.0;
  double a = currentPrice * exp( nu * deltaT );
  double b = sigma * sqrt(deltaT);
  vector<double> spots = partitiate(J_H);
  int length = spots.size()+1;
  double c[length+1];

  c[0]=0;
  c[length]=1;
  for(int i=1;i<length;i++)
    {
      c[i]=cumulative(spots[i-1] - b);
    }

  for(int i=0;i<length;i++)
    underlying.push_back( a * exp( pow(b,2)/2 ) * length * ( c[i+1] - c[i]));
  
  return underlying;
}
// END code1

vector<double> Stock::oneStepUnderlying2(int J_H,double currentPrice,double deltaT){
  vector<double> underlying,rpvector = rp(J_H,deltaT);
  int length = rpvector.size();
  double nu = r - pow(sigma,2.0)/2.0;
  double factor = currentPrice * exp( nu * deltaT ) * length;

  for(int i=0;i<length;i++)
    underlying.push_back( factor * rpvector[i]);
  return underlying;
}
//multi step
vector<double> Stock::haarUnderlying(int J_H,vector<double> times){
  vector<double> rp1,rp2;
  vector<double> underlying;
  if(times.size() < 2){
    cout<<"No time period error"<<endl;
    exit(1);
  }

  rp1 = rp(J_H,times[1] - times[0]);
  for(int i=2; i < times.size(); i++){
    rp2 = rp(J_H, times[i] - times[i-1]);
    rp1 = matrixProduct( rp1, rp2);
  }

  int length = rp1.size();
  double nu = r - pow(sigma,2.0)/2.0;
  double factor = St0 * exp( nu * times[times.size()-1] ) * length;

  for(int i=0;i<length;i++)
    underlying.push_back( factor * rp1[i]);

  return underlying;
}

vector<double> Stock::rp(int J_H,double deltaT){
  vector<double> spots = partitiate(J_H);
  int length = spots.size()+1;

  vector<double> rp;

  for(int i=0;i<length;i++)
    {
      if(i==0)
	rp.push_back(cumulative(spots[i] - sigma * sqrt(deltaT)));
      else if(i==length-1)
	rp.push_back(1 - cumulative(spots[i-1] - sigma * sqrt(deltaT)));
      else
	rp.push_back(cumulative(spots[i] - sigma * sqrt(deltaT)) 
		     - cumulative(spots[i-1] - sigma * sqrt(deltaT)));
    }
  return rp;
}


Path::Path(vector<double> thePrices,vector<double> theJumps,vector<double> theTimes){
  prices = thePrices;
  allJumps = theJumps;
  times = theTimes;
}

// BEGIN code3
Path Path::subPath(  vector<double> subtimes ){
  vector<double> thePrices,theJumps;
  double jump; int j,m=0; 

  for(int i=0;i<subtimes.size();i++){
    jump = 0.0; 
    for(j=m;j<times.size();j++){
      jump += allJumps[j];
      if(times[j] == subtimes[i]){
	thePrices.push_back(prices[j]);
	theJumps.push_back(jump/sqrt(j-m+1.0)); 
	break;
      }
    }
    m = j+1;
  }
  return Path(thePrices,theJumps,subtimes);
}
// END code3

int Path::timeIndex(double time){
  for(int j=0;j<times.size();j++){
    if(times[j] == time)
      return j;
  }
  cout<<"Input error: A valid assess time is an element of all 3 times vector!."<<endl;
  exit(0); //compute 
}
