//bspTest.c
#include "option.h"
#include "stock.h"
#include "portfolio.h"
#include "plotting.h"
#include "falgorithm.h"

#include <cmath>
#include <iostream>
#include <vector>
using namespace std;

// BEGIN code
int main(int argc, char **argv){
  double St0=21.0,interest=0.2,sigma=0.2;
  double time1=1,time2=0.5,strikePrice=21.0,amount=1.0;
  int seed=3;
  vector<double> times;
  for(int i = 0; i<=10;i++)
    times.push_back(i*0.1);

  Stock stock(St0, interest, sigma);
  EuCall ec(stock, time1, strikePrice, amount);
  EuPut ep(stock, time2, strikePrice, amount);  
  
  Path path = stock.simulatePath(times,seed);

  cout<<"Times are:  ";
  for(int i =0; i<times.size();i++)
    cout<<times[i]<<"   ";
  cout<<endl;

  cout<<"Path is:  ";
  for(int i =0; i<path.prices.size();i++)
    cout<<path.prices[i]<<"   ";
  cout<<endl;

  vector<EuCall> ecs;
  vector<EuPut> eps;

  ecs.push_back(ec);
  //eps.push_back(ep);

  Portfolio p(ecs,eps);

  vector<double> bsps = p.sampleBSPortfolio(path);
  vector<double> hdps = p.sampleHoldingPortfolio(path);
  vector<double> hhps = p.sampleHHPayoff(path,10,1);
  
  vector<double> rebalanceTimes;
  for(int i=0;i<times.size();i++)
    rebalanceTimes.push_back(times[i]);    

  cout<<bsps.size()<<"   "<<hdps.size()<<"   "<<hhps.size()<<endl;
  for(int i =0; i<bsps.size();i++)
    cout<<bsps[i]<<"   "<<hdps[i]<<"   "<<hhps[i]<<endl;

  vector<double> xvector = rebalanceTimes;
  vector< vector<double> > yvectors;

  int option=0;
  if(argc==2)
    option=atoi(argv[1]);
  if(option==0){
    char *names[20];
    names[0] = "BS portfolio";
    names[1] = "Holding Porfolio";
    names[2] = "HH Payoff";
    
    yvectors.push_back(bsps);
    yvectors.push_back(hdps);
    yvectors.push_back(hhps);

    vectorPlotting(yvectors,xvector,names);
  }
  else {
    vector<double> v1,v2;
    double hherror=0,bserror=0;
    for(int i=0;i<hdps.size();i++){
      v1.push_back(bsps[i]-hdps[i]);
      v2.push_back(hhps[i]-hdps[i]);
      bserror += pow(bsps[i]-hdps[i],2.0);
      hherror += pow(hhps[i]-hdps[i],2.0);
    }
    cout<<"Average BSP Error = "<<sqrt(bserror/hdps.size())<<endl;
    cout<<"Average HHP Error = "<<sqrt(hherror/hdps.size())<<endl;
    char *names[20];
    names[0] = "BSP Error";
    names[1] = "HHP Error";
    yvectors.push_back(v1);
    yvectors.push_back(v2);
    vectorPlotting(0,yvectors,xvector,names);
  }
  return 0;
}
// END code
